<?php
/**
 * Created by PhpStorm.
 * User: rafaelporto
 * Date: 28/01/2016
 * Time: 11:22
 */

namespace Service;

use Doctrine\ORM\EntityManager;
use Exception\CustomException;
use  Util\DevOpsEnvironment AS DevOpsEnvironment;
use Util\InstanceProvider;
use Util\Utilities;
use Teknisa\Libs\Util\Utilities as LibsUtilities;
use Zeedhi\Framework\Log\Exception;


class CustomizationFiles
{
    /** @var EntityManager $entityManager */
    protected $entityManager;
    /** @var \Util\Utilities */
    private $environment;

    /** @var \Teknisa\Model\Teknisa\Repositories\Licencauso $licencausoRepository */
    protected  $licencausoRepository;

    /** @var \Teknisa\Model\Teknisa\Repositories\Linpluso $useLicenseRepository */
    protected  $useLicenseRepository;

    /** @var \Teknisa\Model\Geral\Repositories\Cliente $clienteRepository */
    protected  $clienteRepository;

    /** @var \Teknisa\Model\Teknisa\Repositories\Reprlegalempr $reprlegalemprRepository */
    protected  $reprlegalemprRepository;

    /** @var \Teknisa\Model\Teknisa\Repositories\Contratolinp $contratolinpRepository */
    protected  $contratolinpRepository;

    /** @var \Teknisa\Model\Geral\Repositories\Pais $countryRepository */
    protected  $countryRepository;

    /** @var \Teknisa\Model\Geral\Repositories\Estado $stateRepository */
    protected  $stateRepository ;

    /** @var \Teknisa\Model\Geral\Repositories\Municipio $stateRepository */
    protected  $municipioRepository ;

    /** @var \Teknisa\Model\Teknisa\Repositories\Caixaluso $useCashRepository */
    protected  $useCashRepository;

    /** @var \Teknisa\Model\Teknisa\Repositories\Imprluso $imprlusoRepository */
    protected  $imprlusoRepository;

    /** @var \Teknisa\Model\Faturamento\Repositories\Tipoimpr $printerTypeRepository */
    protected  $printerTypeRepository;

    /** @var \Teknisa\Model\Geral\Repositories\Organizacao $organizacaoRepository */
    protected  $organizacaoRepository;

    /** @var DevOpsEnvironment */
    private $devOpsEnvironment;

    /** @var \Factory\CustomizationFiles $productCallService */
    protected $customizationFilesFactory;

    /** @var \Teknisa\Model\Geral\Repositories\Indice  */
    protected $indiceRepository;

    public function __construct(EntityManager $entityManager, \Util\Utilities $environment, \Zeedhi\DevOps\Util\Utilities $devOpsEnvironment,
                                \Factory\CustomizationFiles $customizationFilesFactory)
    {
        $this->entityManager = $entityManager;
        $this->licencausoRepository = $this->entityManager->getRepository('\Teknisa\Model\Teknisa\Entities\Licencauso');
        $this->useLicenseRepository = $this->entityManager->getRepository('\Teknisa\Model\Teknisa\Entities\Linpluso');
        $this->indiceRepository = $this->entityManager->getRepository('\Teknisa\Model\Geral\Entities\Indice');
        $this->reprlegalemprRepository = $this->entityManager->getRepository('\Teknisa\Model\Teknisa\Entities\Reprlegalempr');
        $this->contratolinpRepository = $this->entityManager->getRepository('\Teknisa\Model\Teknisa\Entities\Contratolinp');
        $this->countryRepository = $this->entityManager->getRepository('\Teknisa\Model\Geral\Entities\Pais');
        $this->stateRepository = $this->entityManager->getRepository('\Teknisa\Model\Geral\Entities\Estado');
        $this->municipioRepository = $this->entityManager->getRepository('\Teknisa\Model\Geral\Entities\Municipio');
        $this->clienteRepository = $this->entityManager->getRepository('\Teknisa\Model\Geral\Entities\Cliente');
        $this->useCashRepository = $this->entityManager->getRepository('\Teknisa\Model\Teknisa\Entities\Caixaluso');
        $this->imprlusoRepository = $this->entityManager->getRepository('\Teknisa\Model\Teknisa\Entities\Imprluso');
        $this->printerTypeRepository = $this->entityManager->getRepository('\Teknisa\Model\Faturamento\Entities\Tipoimpr');
        $this->organizacaoRepository = $this->entityManager->getRepository('\Teknisa\Model\Geral\Entities\Organizacao');
        $this->customizationFilesFactory = $customizationFilesFactory;
        $this->environment = $environment;
        $this->devOpsEnvironment = $devOpsEnvironment;
    }

    public function findCustomizationFiles($params, $page, $pageSize,$orderBy) {

        if($orderBy){
            if(key($orderBy) == "DTVALIPERSLUSO" || key($orderBy) == "DTLIBEPERSLUSO"){
                $orderBy = "to_date(" . key($orderBy). ",'dd/mm/yyyy') " . current($orderBy);
            }else{
                $this->orderReverse = current($orderBy) == 'ASC';
                $orderByName        = key($orderBy);
                $this->orderBy      = $orderByName;
                $orderBy            = $orderByName ." ". current($orderBy);
            }

        } else {
            $orderBy = "CDLOCALUSO ASC";
        }
        $orderBy .= " NULLS LAST";
        $filter = $this->devOpsEnvironment->createQueryFiltered($params);
        $cdCliente = $this->environment->getCurrentCdCliente();
        $customizationFiles = $this->licencausoRepository->getCustomizationFiles($cdCliente, $filter, $page, $pageSize,$orderBy);
        foreach($customizationFiles as &$customizationFile) {
            $customizationFile['CDLINPCLIE'] = $this->devOpsEnvironment->setSelectMultiInQuery($customizationFile['CDLINPCLIE']);
        }
        return $customizationFiles;
    }

    public function findPerValidationDate(){
        $cdCliente = $this->environment->getCurrentCdCliente();
        $validation = false;
        $userClient = $this->environment->getCurrentUser()->getCdcliente();
        /** @var $cliente  \Teknisa\Model\Geral\Entities\Cliente */
        $cliente = $this->clienteRepository->findOneBy(array('cdcliente' => $cdCliente));
        if($userClient != $cdCliente && Utilities::currentUserIsTeknisa()){
            $validation = 'CLIENT';
        }else if($cliente->getIdlibgerper() == 'S'){
            if($cliente->getDtvalarqpers()){
                $validation = $cliente->getDtvalarqpers()->format('d-m-Y');
            }else if($cliente->getNrdiavalpers()){
                $validation = new \DateTime();
                $validation = $validation->modify($cliente->getNrdiavalpers().' days');
                $validation = $validation->format('d-m-Y');
            }
        }
        return [$validation];
    }

    public function findClientMoney(){
        $cdCliente = $this->environment->getCurrentCdCliente();
        /** @var $cliente  \Teknisa\Model\Geral\Entities\Cliente */
        $cliente = $this->clienteRepository->findOneBy(array('cdcliente' => $cdCliente));
        return [$this->getIndiceByClient($cliente)];
    }

    /**
     * @param \Teknisa\Model\Geral\Entities\Cliente $cliente
     * @return  null |\Teknisa\Model\Geral\Entities\Indice
     */

    private function getIndiceByClient(\Teknisa\Model\Geral\Entities\Cliente $cliente = null)
    {
        $cdmoeda = 'real';

        if ($cliente->getCdindice() != null) {
            $id = array(
                "cdtpindice" => $cliente->getCdtpindice(),
                "cdindice" => $cliente->getCdindice(),
                "idpervalor" => $cliente->getIdpervalor(),
                "nrorg" => $cliente->getNrorg(),
            );
            $cdmoeda = $this->indiceRepository->findOneBy($id)->getNmindice();

        }
        return $cdmoeda;
    }

    public function findLegalRepresentative($params) {
        $filter = $this->devOpsEnvironment->createQueryFiltered($params);
        $cdCliente = $this->environment->getCurrentCdCliente();
        $legalRepresentative = $this->reprlegalemprRepository->findLegalRepresentative($filter, $cdCliente);
        return $legalRepresentative;
    }

    public function findUseLicense($params) {
        $filter = $this->devOpsEnvironment->createQueryFiltered($params);
        $cdCliente = $this->environment->getCurrentCdCliente();
        return $this->useLicenseRepository->getUseLicense($cdCliente, $filter);
    }

    public function findCountry($filters) {
        return $this->countryRepository->obtemTodosPaises($filters);
    }

    public function findCity($sgEstado, $filters, $countryId) {
        return  InstanceProvider::getRepository()->getLocalMunicipio()->getCity($sgEstado, $countryId, $filters);
    }

    public function findPrinterType($filters) {
        return $this->printerTypeRepository->getPrinterType($filters);
    }

    public function findState($countryId, $filters) {
        return $this->stateRepository->getStateByCountry($countryId, $filters);
    }

    public function findCash($licencaUsoId) {
        $cdCliente = $this->environment->getCurrentCdCliente();
        return $this->useCashRepository->getCashBylicencauso($licencaUsoId, $cdCliente);
    }

    public function validateCaixa($licencaUsoIds, $IdtipLinp) {
        $cdCliente = $this->environment->getCurrentCdCliente();
        $licencaUsoArray = [];
        foreach($licencaUsoIds as $key=>$licencaUso){
            if($IdtipLinp[$key] == 'P' || $IdtipLinp[$key] == 'A'){
                $licensasUso = '%'.Utilities::zeroLeftPadding($licencaUso, 10).'%';


                $filter =" AND NRLICENCAUSO LIKE '" . $licensasUso . "'";

                $cash = $this->useCashRepository->validateCaixa($filter, $cdCliente);
                if(!$cash){
                    $licencaUsoArray[] = $licencaUso;
                }
            }

        }
        if($licencaUsoArray){
           return $licencaUsoArray;
        }else{

            return false;
        }

    }

    public function validateUseLicense($licencauso) {
         $cdcliente = $this->environment->getCurrentCdCliente();
         $cliente = $this->clienteRepository->findOneBy(array('cdcliente' => $cdcliente));
         $nrparcnegocio = $cliente->getNrparcnegocio();
         $org = InstanceProvider::getRepository()->getLocalOrganizacao()->getOrganizacao($nrparcnegocio);
         $nrorg = empty($org['NRORGPRINCIPAL']) ? $org['NRORG'] : $org['NRORGPRINCIPAL'];
         $licensasUsoEmUso = $this->organizacaoRepository->findOrganizationsAndTheirRespectiveBranches($nrorg);

         $orgInfo = false;
         if($licencauso && $licensasUsoEmUso){
            foreach($licensasUsoEmUso as $licUsoEmUso){
                    if(strcmp($licUsoEmUso['FILIAL'], $licencauso) == 0){
                         $orgInfo["NMORGANIZACAO"] =  $licUsoEmUso["NMORGANIZACAO"];
                         $orgInfo["NMRAZSOCIALORG"] =  $licUsoEmUso["NMRAZSOCIALORG"];
                         $orgInfo["NRORG"] =  $licUsoEmUso["NRORG"];
                         $orgInfo["NRLOCALUSO"] =  $licencauso;
                         if($licUsoEmUso["NRORG"] == $org['NRORG'] ){
                             $orgInfo["SAME_ORG"] = false;
                         }else{
                             $orgInfo["SAME_ORG"] = true;
                         }
                     }
             }
         }
         return $orgInfo;

    }

    public function findPrinter($licencaUsoId) {
        $cdCliente = $this->environment->getCurrentCdCliente();
        return $this->imprlusoRepository->getPrinter($licencaUsoId, $cdCliente);
    }

    public function saveCash($cdCaixa, $nmCaixa, $licencauso) {
        $connection = $this->entityManager->getConnection();
        try {
            $this->entityManager->beginTransaction();
            $newCash = $this->customizationFilesFactory->createCash($cdCaixa, $nmCaixa, $licencauso, $this->environment->getCurrentCdCliente());
            $this->entityManager->persist($newCash);
            $this->entityManager->flush();
            $connection->commit();
        } catch(\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function savePrinter($nrserimpluso, $nrecfimpluso, $cdtipoimpr, $licencauso) {
        $connection = $this->entityManager->getConnection();
        try {
            $this->entityManager->beginTransaction();
            $newPrinter = $this->customizationFilesFactory->createPrinter($nrserimpluso, $nrecfimpluso, $cdtipoimpr, $licencauso,
                $this->environment->getCurrentOrganizationId(), $this->environment->getCurrentCdCliente());
            $this->entityManager->persist($newPrinter);
            $this->entityManager->flush();
            $connection->commit();
        } catch(\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function saveCustomizationFiles($cdlocaluso, $nmfantluso, $nmrazsocluso, $nrinsjurluso,
                                           $nrinsestluso, $nrinsmuniluso, $nmcontluso, $nmdeptoluso, $idfaturafilisep, $dsendeluso, $nmbairluso,
                                           $nrcepluso, $cdpais, $sgestado, $cdmunicipio, $nrteleluso, $nrramalluso, $nrfaxluso, $dsemailcomlu,
                                           $cdlinpclie, $nrlicencauso, $idmodproc, $idlocaproc, $isNew, $idlusoativ, $idtpijurluso) {
        $connection = $this->entityManager->getConnection();
        $cdcliente = $this->environment->getCurrentCdCliente();
        try {
            $this->entityManager->beginTransaction();
            /** @var  \Teknisa\Model\Teknisa\Entities\Licencauso $customizationFile*/
            $customizationFile = $this->licencausoRepository->findOneBy(array('cdcliente' => $cdcliente, 'cdlocaluso' => $cdlocaluso));
            if($isNew) {
                if (!$customizationFile) {
                    $customizationFile = $this->customizationFilesFactory->saveLicencauso($cdlocaluso, $nmfantluso, $nmrazsocluso, $nrinsjurluso,
                        $nrinsestluso, $nrinsmuniluso, $nmcontluso, $nmdeptoluso, $idfaturafilisep, $dsendeluso, $nmbairluso,
                        $nrcepluso, $cdpais, $sgestado, $cdmunicipio, $nrteleluso, $nrramalluso, $nrfaxluso, $dsemailcomlu,
                        $cdcliente, $nrlicencauso, $idmodproc, $idlocaproc, false, 'S', 1, $idtpijurluso);
                }else{
                    $idAtivo = $customizationFile->getIdlusoativ() == 'S' ? 'Ativo' : 'Inativo';
                    throw new \Exception('Inclusão bloqueada! Existe uma Licença de Uso já cadastrada com o código '.$cdlocaluso.'<br/>('.$customizationFile->getNmfantluso().' - '.$idAtivo.')');
                }
            }else{
                $customizationFile = $this->customizationFilesFactory->saveLicencauso($cdlocaluso, $nmfantluso, $nmrazsocluso, $nrinsjurluso,
                    $nrinsestluso, $nrinsmuniluso, $nmcontluso, $nmdeptoluso, $idfaturafilisep, $dsendeluso, $nmbairluso,
                    $nrcepluso, $cdpais, $sgestado, $cdmunicipio, $nrteleluso, $nrramalluso, $nrfaxluso, $dsemailcomlu,
                    $cdcliente, $nrlicencauso, $idmodproc, $idlocaproc, $customizationFile, $idlusoativ, 1, $idtpijurluso);
            }
            $productLineRelatedToUseLicenses = $this->useLicenseRepository->findBy(array('cdcliente' => $cdcliente, 'nrlicencauso' => $nrlicencauso));
            $productLineRelatedToUseLicenseIdArray = array();
            foreach($productLineRelatedToUseLicenses as $productLineRelatedToUseLicense){
                if(!in_array($productLineRelatedToUseLicense->getCdlinpclie(), $cdlinpclie)){
                    $this->entityManager->remove($productLineRelatedToUseLicense);
                }else{
                    array_push($productLineRelatedToUseLicenseIdArray, $productLineRelatedToUseLicense->getCdlinpclie());
                }
            }
            if($cdlinpclie){
                foreach($cdlinpclie as $userLicenseId) {
                    if (!in_array($userLicenseId, $productLineRelatedToUseLicenseIdArray)) {
                        $linpluso = $this->customizationFilesFactory->createLinpluso($cdcliente, $nrlicencauso, $userLicenseId);
                        $this->entityManager->persist($linpluso);
                    }
                }
            }


            $this->entityManager->persist($customizationFile);
            $this->entityManager->flush();
            $connection->commit();
        } catch(\Exception $e) {
            $connection->rollBack();

            if(strpos($e->getMessage(), 'ORA-2'))
            {
                $message = explode('ORA-' , $e->getMessage())[1];
                $message = explode(':' , $message)[1];
                throw new CustomException($message);
            }
            throw $e;
        }
    }

    public function saveLegalRepresentative($cdreprlegalempr,$nmreprlegalempr, $nrrgreprlegal, $cdexrgreprlegal , $nrtelreprlegal,
                                            $dsendereprlegal, $nmbairreprlegal, $nrcepreprlegal, $cdpais, $sgestado, $cdmunicipio, $nrfaxreprlegal, $nrcelreprlegal, $dsemailreprlegal,
                                            $licensauso, $search = false){
        $connection = $this->entityManager->getConnection();
        $cdcliente = $this->environment->getCurrentCdCliente();
        try {
            $this->entityManager->beginTransaction();
            /** @var  \Teknisa\Model\Teknisa\Entities\Licencauso $customizationFile*/
            $customizationFile = $this->licencausoRepository->findOneBy(array('cdcliente' => $cdcliente, 'cdlocaluso' => $licensauso));
            $customizationFile->setCdreprlegalempr($cdreprlegalempr);
            if(!$search) {

                /** @var  \Teknisa\Model\Teknisa\Entities\Reprlegalempr $legalRepresentative */
                $legalRepresentative = $this->reprlegalemprRepository->findOneBy(array('cdreprlegalempr' => $cdreprlegalempr));

                if ($legalRepresentative) {
                    $this->customizationFilesFactory->editLegalRepresentative($legalRepresentative, $nmreprlegalempr, $nrrgreprlegal, $cdexrgreprlegal, $nrtelreprlegal,
                        $dsendereprlegal, $nmbairreprlegal, $nrcepreprlegal, $cdpais, $sgestado, $cdmunicipio, $nrfaxreprlegal, $nrcelreprlegal, $dsemailreprlegal);
                } else {
                    $legalRepresentative = $this->customizationFilesFactory->createLegalRepresentative($cdreprlegalempr, $nmreprlegalempr, $nrrgreprlegal, $cdexrgreprlegal, $nrtelreprlegal,
                        $dsendereprlegal, $nmbairreprlegal, $nrcepreprlegal, $cdpais, $sgestado, $cdmunicipio, $nrfaxreprlegal, $nrcelreprlegal, $dsemailreprlegal);

                }
                $this->entityManager->persist($legalRepresentative);
            }
            $this->entityManager->persist($customizationFile);
            $this->entityManager->flush();
            $connection->commit();
        } catch(\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function removeCash($cdCaixa, $licencauso) {
        $connection = $this->entityManager->getConnection();
        try {
            $cdCliente  = $this->environment->getCurrentCdCliente();
            $connection->beginTransaction();
            $cash = $this->useCashRepository->findOneBy(array('cdcaixaluso' => $cdCaixa, 'cdcliente' => $cdCliente, 'nrlicencauso' => $licencauso));
            $this->entityManager->remove($cash);

            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function removePrinter($nrserimpluso, $licencauso) {
        $connection = $this->entityManager->getConnection();
        try {
            $cdCliente  = $this->environment->getCurrentCdCliente();
            $connection->beginTransaction();
            $printer = $this->imprlusoRepository->findOneBy(array('nrserimpluso' => $nrserimpluso, 'cdcliente' => $cdCliente, 'nrlicencauso' => $licencauso));
            $this->entityManager->remove($printer);

            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function removeCustomizationFile($licencauso) {
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            $cdCliente  = $this->environment->getCurrentCdCliente();
            /** @var  \Teknisa\Model\Teknisa\Entities\Licencauso $customizationFile*/
            $customizationFile = $this->licencausoRepository->findOneBy(array('cdcliente' => $cdCliente, 'nrlicencauso' => $licencauso));
            $customizationFile->setIdlusoativ('N');
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function getLimSemPer($licencauso) {
        try {

            $now = new \DateTime();
            $now = $now->format('d/m/Y');
            $licencauso = implode("chk", $licencauso);
            $licencauso = "chk".$licencauso;
            $cdOperador = $this->environment->getCurrentUserId();
            $cdCliente = $this->environment->getCurrentCdCliente();

            $url = Utilities::getUrlWSDLSecurityTeknisa();

            $options = Utilities::getOptionsSoap();

            $args = array(
                new \SoapVar($cdCliente, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema"),
                new \SoapVar($licencauso, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema"),
                new \SoapVar($cdOperador, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema"),
                new \SoapVar(' ', XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema"),
                new \SoapVar(' ', XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema"),
                new \SoapVar($now, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema")
            );

            $params = array(
                'rdm'    =>  "FAS.rdmLicencaUso",
                'method' => "GetLinSemPer",
                'args'   => $args
            );

            $client = new \SoapClient($url, $options);
            $result = $client->executeCOMMethod($params);
            $result = explode("@@", $result->executeCOMMethodResult);
            if(empty($result[1]) || empty($result[0])){
                throw new CustomException('ALERT_PER_BLOCKED_NOT_LINE_1');
            }
            $resultArray = array('listaFili' => $result[0], 'listaTrab' => $result[1]);
            $clientId = $this->environment->getCurrentCdCliente();
            $validate = $this->licencausoRepository->validateLinpluso($clientId, $result[0]);
            if(!$validate){
                throw new CustomException('ALERT_PER_BLOCKED_NOT_LINE');
            }
            return $resultArray;
        } catch (\SoapFault $ex) {
            throw  $ex;
        }
    }

    public function recordContatoPer() {
        try {
            $cdOperador = $this->environment->getCurrentUserId();
            $cdCliente = $this->environment->getCurrentCdCliente();

            $url = Utilities::getUrlWSDLSecurityTeknisa();

            $options = Utilities::getOptionsSoap();

            $args = array(
                new \SoapVar($cdCliente, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema"),
                new \SoapVar($cdOperador, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema"),
            );

            $params = array(
                'rdm'    =>  "SRV.rdmContato",
                'method' => "GravaContatoPer",
                'args'   => $args
            );

            $client = new \SoapClient($url, $options);
            $result = $client->executeCOMMethod($params);
            return $result;
        } catch (\SoapFault $ex) {
            throw  $ex;
        }
    }
    public function findLista($listaFili){
        return InstanceProvider::getRepository()->getLocalLista()->findLista($listaFili);
    }

    public function generateNewPer($listaFili, $listaTrab, $nrcontatoper, $endMail, $idIdioma, $endDate, $parcial = false) {
        try {
            ini_set('default_socket_timeout', 7200);
            ini_set('max_execution_time', '7200');
            set_time_limit(7200);
            $cdOperador = $this->environment->getCurrentUserId();
            $userIp = LibsUtilities::getCurrentIp();
            $cdCliente = $this->environment->getCurrentCdCliente();
            $cdMoeda = $this->getIndiceByClient($this->clienteRepository->findOneBy(array('cdcliente' => $cdCliente)));
            $url = Utilities::getUrlWSDLSecurityTeknisa();

            $options = Utilities::getOptionsSoap();

            if($parcial){
                $listaFili = 'PARCIAL_'.$listaFili;
            }

            $args = array(
                new \SoapVar($cdCliente, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema"),
                new \SoapVar($listaFili, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema"),
                new \SoapVar($listaTrab, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema"),
                new \SoapVar($endDate, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema"),
                new \SoapVar($cdMoeda, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema"),
                new \SoapVar($cdOperador, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema"),
                new \SoapVar($nrcontatoper, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema"),
                new \SoapVar($endMail, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema"),
                new \SoapVar($userIp, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema"),
                new \SoapVar($idIdioma, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema")
            );

            $params = array(
                'rdm'    =>  "FAS.rdmLicencaUso",
                'method' => "geraPerNovo",
                'args'   => $args
            );
            $client = new \SoapClient($url, $options);
            $result = $client->executeCOMMethod($params);
            return $result;
        } catch (\SoapFault $ex) {
            throw  $ex;
        }
    }

    public function delListaTrab($listaFili, $listaTrab) {
        try {
            $cdCliente = $this->environment->getCurrentCdCliente();
            $url = Utilities::getUrlWSDLSecurityTeknisa();
            $options = Utilities::getOptionsSoap();

            $args = array(
                new \SoapVar($cdCliente, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema"),
                new \SoapVar($listaFili, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema"),
                new \SoapVar($listaTrab, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema"),
            );

            $params = array(
                'rdm'    =>  "FAS.rdmLicencaUso",
                'method' => "DelListaTrab",
                'args'   => $args
            );

            $client = new \SoapClient($url, $options);
            $result = $client->executeCOMMethod($params);
            return $result;
        } catch (\SoapFault $ex) {
            throw  $ex;
        }
    }

    public function sendSatEmail($cdluso, $nmfant, $email, $cnpj) {
        try {
            $cdCliente = $this->environment->getCurrentCdCliente();
            $url = Utilities::getUrlWSDLSecurityTeknisaSAT();
            $options = Utilities::getOptionsSoap();

            $args = array(
                new \SoapVar($cdCliente, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema"),
                new \SoapVar($cdluso, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema"),
                new \SoapVar($nmfant, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema"),
                new \SoapVar($cnpj, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema"),
                new \SoapVar($email, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema"),
            );

            $params = array(
                'rdm'    =>  "FAS.rdmLicencaUso",
                'method' => "EnviaSatEmail",
                'args'   => $args
            );

            $client = new \SoapClient($url, $options);
            $result = $client->executeCOMMethod($params);
            return $result;
        } catch (\SoapFault $ex) {
            throw  $ex;
        }
    }

    public function findCostsPer($cdClient, $page, $pageSize, $conditions)
    {
        $limit = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        $filter = Utilities::createQueryFiltered($conditions);
        return InstanceProvider::getRepository()->getLinpuso()->findCostsPer($cdClient, $limit, $offset, $filter);
    }

    public function findCostsPerGroupedFilial($cdClient, $page, $pageSize, $conditions)
    {
        $limit = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        $filter = Utilities::createQueryFiltered($conditions);
        return InstanceProvider::getRepository()->getLinpuso()->findCostsPerGroupedFilial($cdClient, $limit, $offset, $filter);
    }

}
