<?php
namespace Zeedhi\DevOps\Model\Projectmanagement\Repositories;


class Profile extends \Doctrine\ORM\EntityRepository {
	
	public function getProfiles($structuresIDs, $organization){
		$query = <<<SQL
			SELECT P.ID, P.LABEL AS NAME, P.CONTROL, COUNT(PP.ID) AS ASSOCIATION_COUNT
			FROM 		PROFILE P
			LEFT JOIN 	PARTNER_PROFILE PP ON PP.PROFILE = P.ID
			WHERE 	P.STRUCTURE IN ($structuresIDs)
			OR 		(P.ORGANIZATION = :organization AND P.STRUCTURE IS NULL)
			AND 	P.CATEGORY <> 'A'
			GROUP BY P.ID, P.LABEL, P.CONTROL
SQL;
		$params = array(
			":structuresIDs" => $structuresIDs,
			":organization" => $organization
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}

	public function getProfile($profile){
		$query = <<<SQL
			SELECT P.ID, P.LABEL AS NAME, P.CATEGORY AS CATEGORY_CONTROL, P.STRUCTURE, S.NAME AS STRUCTURE_NAME,
				   P.CONTROL AS CONTROL
			FROM 		PROFILE P
			LEFT JOIN 	STRUCTURE S ON S.ID = P.STRUCTURE
			WHERE P.ID = :profile
SQL;
		$params = array(
			":profile" => $profile
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
	}

	public function getProfileByName($categoryName, $organizationId){
		$query = <<<SQL
			SELECT P.ID, P.CATEGORY, P.STRUCTURE, S.NAME AS STRUCTURE_NAME, P.LABEL
				FROM PROFILE P
				INNER JOIN CATEGORY C ON C.ID = P.CATEGORY
				LEFT JOIN STRUCTURE S ON S.ID = P.STRUCTURE
				WHERE 	P.ORGANIZATION = :organization
				AND 	C.CONTROL = :category
SQL;
		$params = array(
			":organization" => $organizationId,
			":category" => $categoryName
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);

	}

	public function getProfileByOrganization($organizationId){
		$query = <<<SQL
			SELECT P.ID, P.CONTROL, P.EXTERNAL_ID, P.LABEL, P.STATUS
			  FROM PROFILE P
             WHERE P.ORGANIZATION = :organization
               AND STATUS <> 'D'
SQL;
		$params = array(
			":organization" => $organizationId
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $params);

	}

    public function findPerfilAcesso($organizationId){
        $query = <<<SQL
			SELECT P.NRPERFILACESSO, P.NMPERFILACESSO
			  FROM PERFILACESSO P
             WHERE P.NRORG = :organization
               AND IDATIVO = 'S'
SQL;
        $params = array(
            ":organization" => $organizationId
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);

    }

	
}